from .accessors.api import *
from .base import Session


class Funkwhale:

    artists = ArtistAccessor()
    albums = AlbumsAccessor()
    tracks = TracksAccessor()
    libraries = LibraryAccessor()
    uploads = UploadsAccessor()
    favourites = FavoritesAccessor()
    playlists = PlaylistAccessor()

    def __init__(self, base_url):
        self.session = Session(base_url)

    async def login(self, username, password):
        async with self.session:
            await self.session.login(username, password)



import argparse
from functools import wraps
import asyncio

from . import Funkwhale


def as_sync(fn):
    @wraps(fn)
    def wrapper(*args, **kwargs):
        res = fn(*args, **kwargs)
        if asyncio.iscoroutine(res):
            return asyncio.get_event_loop().run_until_complete(res)
        return res
    return wrapper


async def main():
    parser = argparse.ArgumentParser()
    args = parser.parse_args()

    api = Funkwhale("https://funkwhale-paftown.ddns.net")
    await api.login("wolfgang", "N@AM4-:W[2S~.B+5")

    async with api.session:
        response = await api.artists.list_libraries(1)
        print(response)


if __name__ == "__main__":
    as_sync(main)()



import os
from enum import Enum

import aiohttp


class Visibility(Enum):
    ME = "me"
    INSTANCE = "instance"
    EVERYONE = "everyone"


class Session:
    """
    This class handles the actual http requests and the authentication
    """

    token: str
    session: aiohttp.ClientSession

    def __init__(self, base_url: str):
        self.base_url = base_url
        self.session = None
        self.token = None

    async def __aenter__(self):
        self.session = aiohttp.ClientSession(
            # timeout=aiohttp.ClientTimeout(total=10),
            headers={"User-Agent": "funkwhale/cli"}
        )
        await self.session.__aenter__()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.session:
            await self.session.__aexit__(exc_type, exc_val, exc_tb)
            self.session = None

    async def login(self, username, password):
        response = await self.session.post(
            f"{self.base_url}/api/v1/token",
            data={"username": username, "password": password}
        )
        if response.status == 400:
            raise RuntimeError("Unable to log in")
        else:
            self.token = (await response.json())["token"]

    async def request(self, method: str, url: str, *args, **kwargs):
        if url.startswith("http://") or url.startswith("https://"):
            full_url = url
        else:
            full_url = self.base_url + url
        headers = kwargs.setdefault("headers", {})
        if self.token:
            scheme = "JWT" if len(self.token) > 50 else "Bearer"
            headers["Authorization"] = f"{scheme} {self.token}"
        response = await getattr(self.session, method)(full_url, *args, **kwargs)
        response.raise_for_status()
        return await response.json()



import os

from ..base import Session


class ApiAccessor:

    session: Session
    url: str

    def __get__(self, instance, owner):
        if instance:
            self.session = instance.session
        return self

    def _endpoint(self, *args):
        return os.path.join(self.url, *map(str, args))


class ListAccessor(ApiAccessor):

    async def list(self, **kwargs):
        return await self.session.request(
            "get", self._endpoint(),
            **kwargs
        )


class GetAccessor(ApiAccessor):

    async def get(self, identifier, **kwargs):
        return await self.session.request(
            "get", self._endpoint(identifier),
            **kwargs
        )


class ListLibrariesAccessor(ApiAccessor):

    async def list_libraries(self, identifier, **kwargs):
        return await self.session.request(
            "get", self._endpoint(identifier, "libraries"),
            **kwargs
        )


class NewAccessor(ApiAccessor):

    async def new(self, **kwargs):
        return await self.session.request(
            "post", self._endpoint(),
            **kwargs
        )


class UpdateAccessor(ApiAccessor):

    async def update(self, identifier, **kwargs):
        return await self.session.request(
            "post", self._endpoint(identifier),
            **kwargs
        )


class DeleteAccessor(ApiAccessor):

    async def delete(self, identifier, **kwargs):
        return await self.session.request(
            "delete", self._endpoint(identifier),
            **kwargs
        )



import os
from datetime import datetime

from funksnake.base import Visibility
from .base import *


# ==================== #
# Library and metadata #
# ==================== #
class ArtistAccessor(ListAccessor, GetAccessor, ListLibrariesAccessor):

    url = "/api/v1/artists"


class AlbumsAccessor(ListAccessor, GetAccessor, ListLibrariesAccessor):

    url = "/api/v1/albums"


class TracksAccessor(ListAccessor, GetAccessor, ListLibrariesAccessor):

    url = "/api/v1/tracks"


# ========================= #
# Uploads and audio content #
# ========================= #
class LibraryAccessor(ListAccessor, GetAccessor, NewAccessor, UpdateAccessor, DeleteAccessor):

    url = "/api/v1/libraries/"

    async def new(self, name: str, visibility: Visibility = Visibility.ME, description: str = None):
        if description:
            description = {"description": description}
        else:
            description = {}

        return await super().new(
            data={
                "name": name,
                "visibility": visibility.value,
                **description
            }
        )


class UploadsAccessor(ListAccessor, GetAccessor, NewAccessor, UpdateAccessor, DeleteAccessor):

    url = "/api/v1/uploads/"

    async def new(self, path, library, ref=None):
        return await super().new(
            data={
                "library": library,
                "import_reference": ref or f"funksnake-import-{datetime.now().isoformat()}",
                "source": f"upload://{os.path.basename(path)}",
                "audio_file": open(path, "rb"),
            },
            timeout=0
        )

    async def metadata(self, identifier):
        return await self.session.request(
            "get", self._endpoint(identifier, "audio-file-metadata")
        )


# ================ #
# Content curation #
# ================ #
class FavoritesAccessor(ListAccessor):

    url = "/api/v1/favorites/tracks/"

    def add(self, track):
        return await self.session.request(
            "post", self._endpoint(),
            data={
                "track": track
            }
        )

    def remove(self, track):
        return await self.session.request(
            "post", self._endpoint("remove"),
            data={
                "track": track
            }
        )


class PlaylistAccessor(ListAccessor, GetAccessor, NewAccessor, UpdateAccessor, DeleteAccessor):

    url = "/api/v1/playlists/"

    async def new(self, name: str, visibility: Visibility = Visibility.ME, **kwargs):
        return await super().new(
            data={
                "name": name,
                "privacy_level": visibility.value
            }
        )

    async def add_tracks(self, identifier: str, tracks: list, allow_duplicates: bool = False):
        return await self.session.request(
            "post", self._endpoint(identifier, "add"),
            data={
                "tracks": tracks,
                "allow_duplicates": allow_duplicates
            }
        )

    async def get_tracks(self, identifier: str):
        return await self.session.request(
            "get", self._endpoint(identifier, "tracks")
        )

    async def move_tracks(self, identifier: str):
        return await self.session.request(
            "post", self._endpoint(identifier, "move")
        )

    async def remove_tracks(self, identifier):
        return await self.session.request(
            "post", self._endpoint(identifier, "remove")
        )

    async def clear_tracks(self, identifier):
        return await self.session.request(
            "delete", self._endpoint(identifier, "clear")
        )



import setuptools


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


setuptools.setup(
    name="funksnake",
    version="0.0.1",
    author="Wolfgang Fischer",
    author_email="31348226+gammelalf@users.noreply.github.com",
    description="Python wrapper for funkwhale api",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/gammelalf/funksnake",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
)
